/*
  This sketch sends data via HTTP GET requests to data.sparkfun.com service.

The destination host is my private daa server, you need your own.
*/

#include <ESP8266WiFi.h>

const char* ssid = "Your.WiFi.SSID";
const char* password = "YourPassword";
const char* host = "192.168.1.2"; // your server address
const unsigned long refresh = 120000; // measure every 2 minutes (in milliseconds)
unsigned long lastread;

void setup() {
  Serial.begin(19200);		// used for Rego
  Serial1.begin(115200);	// used for debug
  delay(10);
  // We start by connecting to a WiFi network
  Serial1.println();
  Serial1.println();
  Serial1.print("Connecting to ");
  Serial1.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial1.print(".");
  }
  Serial1.println("");
  Serial1.println("WiFi connected");
  Serial1.println("IP address: ");
  Serial1.println(WiFi.localIP());
  lastread=millis() - refresh;
}

int16_t read2int(byte *ReadArray) // convert received values (array) to integer
{
  int16_t res = ( (int16_t)*(ReadArray) << 14 ) | ( (int16_t)*(ReadArray+1) << 7 ) | (int16_t)*(ReadArray+2);
  return res;
}

float read2temp(byte *ReadArray) // convert received values (array) to temperature
{
  int16_t res = ( (int16_t)*(ReadArray) << 14 ) | ( (int16_t)*(ReadArray+1) << 7 ) | (int16_t)*(ReadArray+2);
  float MyTemp=0.1 * (float)res;
  return MyTemp;
}

void int2write (int16_t MyVal, byte *WriteArray) // convert int (16 bit) to array for sending
{
  *(WriteArray+2) = (byte)MyVal & 0x7F;
  *(WriteArray+1) = (byte)(MyVal>>7) & 0x7F;
  *WriteArray = (byte)(MyVal>>14) & 0x03;
}

void cmdsend(byte request[9])
{
  //Serial1.print("Write command: ");
  for (int i=0; i< 9; i++)
  {
    Serial.write(request[i]);
    //Serial1.print(request[i],HEX);
    //Serial1.print(" ");
  }
  //Serial1.println("");
}

int cmdread(int max, int timeout, byte *response)
{
  int TO = 0;	// timeout value
  int cnt = 0;	// count of received characters
  boolean done=false;	// are we done with reading?
  do {
    if (Serial.available() > 0) {
      // read the incoming byte:
      *(response+cnt) = Serial.read();
      Serial1.print("R: ");
      Serial1.println(*(response+cnt),HEX);
      if (++cnt == max) {done = true;}
    } else {
      delay(100);
      if (++TO > timeout) {done = true;}
      //Serial1.print("T");
    }
  } while (!done);
  return cnt;
}

void ComposeCmd (byte addr, byte cmd, int16_t reg, int16_t val, byte *request)
{
  *request = addr;
  *(request+1) = cmd;
  *(request+8) = 0; // XOR
  int2write(reg,request+2);
  int2write(val,request+5);
  for (int i=2; i< 8; i++)
  {
     *(request+8) ^= *(request+i); // update XOR with data
  }
}

void SendCmd (byte *request)
{
  //Serial1.print("Command to send: "); // send data to port
  for (int i=0; i< 9; i++)
  {
     Serial.write(*(request+i)); // send data to port
     //Serial1.print(*(request+i),HEX); // send data to port
     //Serial1.print(" "); // send separator
  }
  //Serial1.println(" "); // end of line
}

int16_t ReadVal()
{
  byte MyResponse[5];
  int len;
  int16_t res;
  boolean valid;
  len=cmdread(5,10,MyResponse);
  res = 0x8001;
  valid = (len == 5);
  if ((valid) && (MyResponse[0]!=0x01)) {valid=false; res = 0x8002;}
  if ((valid) && (MyResponse[4]!=(MyResponse[1]^MyResponse[2]^MyResponse[3]))) {valid=false; res = 0x8003;}
  if (valid) {res = read2int(MyResponse+1);}
  return res;
} 

int16_t CmdRespTemp(byte addr, byte cmd, int16_t reg, int16_t val)
{
  byte request[9];
  ComposeCmd(addr, cmd, reg, val, request);
  SendCmd(request);
  return ReadVal();
}

String ReadValue(int16_t reg, String name)
{
  uint16_t prevedeno =0;
  prevedeno=CmdRespTemp(0x81,0x02,reg,0x00);
  if ((prevedeno>=0x8000) && (prevedeno<=0x800F)) {prevedeno=CmdRespTemp(0x81,0x02,reg,0x00);} // if error, try again
  String url = String(reg,HEX);
  url += "=";
  url += String(prevedeno,HEX);
  Serial1.print(name + " ");
  Serial1.print(reg,HEX);
  Serial1.print("=");
  Serial1.print(prevedeno,DEC);
  Serial1.print(" (");
  Serial1.print(prevedeno,HEX);
  Serial1.println(" hex)");
  return url;
}

void SubmitUrl(String url)
{
  Serial1.print("connecting to ");
  Serial1.println(host);
  // Use WiFiClient class to create TCP connections
  WiFiClient client;
  const int httpPort = 80;
  if (!client.connect(host, httpPort)) {
    Serial1.println("connection failed");
    return;
  }
  // We now create a URI for the request
  Serial1.print("Requesting URL: ");
  Serial1.println(url);
  // This will send the request to the server
  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + host + "\r\n" +
               "User-Agent: ESP-001 IVT Rego600\r\n" +
               "Connection: close\r\n\r\n");
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial1.println(">>> Client Timeout !");
      client.stop();
      return;
    }
  }
  // Read all the lines of the reply from server and print them to Serial
  while (client.available()) {
    String line = client.readStringUntil('\r');		// read host output...
    //Serial1.print(line);
  }
  //Serial1.println();
  //Serial1.println("closing connection");
  client.stop();
}

int value = 0;
void loop() {
  if (millis() - lastread < refresh) {	// Time for new measure?
    delay(2);		// no - wait a moment
  } else {
    lastread += refresh;	// yes - do measurement
    String url = "/cgi-bin/ivt.pl?";
    url += ReadValue(0x209,"Zpatecka");
    url += "&";
    url += ReadValue(0x20A,"Venkovni");
    url += "&";
    url += ReadValue(0x20B,"Bojler");
    //url += "&";
    //url += ReadValue(0x20C,"Vstup do smes. okruhu");
    url += "&";
    url += ReadValue(0x20D,"Mistnost");
    url += "&";
    url += ReadValue(0x20E,"Kompresor");
    url += "&";
    url += ReadValue(0x20F,"Teply Vystup");
    url += "&";
    url += ReadValue(0x210,"Teply Vstup");
    url += "&";
    url += ReadValue(0x211,"Studeny Vstup");
    url += "&";
    url += ReadValue(0x212,"Studeny Vystup");
    SubmitUrl(url);
  
    url = "/cgi-bin/ivt.pl?";
    url += ReadValue(0x1FD,"Zemni cerpadlo [P3]");
    url += "&";
    url += ReadValue(0x1FE,"Kompresor");
    url += "&";
    url += ReadValue(0x1FF,"Pridavne topeni 3kW");
    url += "&";
    url += ReadValue(0x200,"Pridavne topeni 6kW");
    url += "&";
    url += ReadValue(0x203,"Radiatorove cerpadlo");
    url += "&";
    url += ReadValue(0x204,"Cerpadlo vnitrniho obehu");
    url += "&";
    url += ReadValue(0x205,"Trojcestny ventil");
    url += "&";
    url += ReadValue(0x206,"Porucha");
    SubmitUrl(url);
  
    url = "/cgi-bin/ivt.pl?RSSI=";
    url += String(WiFi.RSSI());
    SubmitUrl(url);
  }
}
